import django

if django.VERSION < (1, 6):
    from .test_backends import *
    from .test_commands import *
    from .test_privacy import *
    from .tests_decorators import *
    from .tests_forms import *
    from .tests_managers import *
    from .tests_middleware import *
    from .tests_models import *
    from .tests_utils import *
    from .tests_views import *


import sys, re, six

from django.test import TestCase
from django.conf import settings
from django.utils.six.moves.urllib_parse import urlparse, parse_qs

from userena.utils import (get_gravatar, signin_redirect, get_profile_model,
                           get_protocol, get_user_model, generate_sha1)
from userena import settings as userena_settings
from userena.compat import SiteProfileNotAvailable


class UtilsTests(TestCase):
    """ Test the extra utils methods """
    fixtures = ['users']

    def test_generate_sha(self):
        s1 = six.u('\xc5se')
        s2 = six.u('\xd8ystein')
        s3 = six.u('\xc6gir')
        h1 = generate_sha1(s1)
        h2 = generate_sha1(s2)
        h3 = generate_sha1(s3)
        # Check valid SHA1 activation key
        self.failUnless(re.match('^[a-f0-9]{40}$', h1[1]))
        self.failUnless(re.match('^[a-f0-9]{40}$', h2[1]))
        self.failUnless(re.match('^[a-f0-9]{40}$', h3[1]))

    def test_get_gravatar(self):
        template = 's=%(size)s&d=%(type)s'

        # Check the defaults.
        parsed = urlparse(get_gravatar('alice@example.com'))
        self.failUnlessEqual(
            parse_qs(parsed.query),
            parse_qs(template % {'size': 80, 'type': 'identicon'})
        )

        # Check different size
        parsed = urlparse(get_gravatar('alice@example.com', size=200))
        self.failUnlessEqual(
            parse_qs(parsed.query),
            parse_qs(template % {'size': 200, 'type': 'identicon'})
        )

        # Check different default
        parsed = urlparse(get_gravatar('alice@example.com', default='404'))
        self.failUnlessEqual(
            parse_qs(parsed.query),
            parse_qs(template % {'size': 80, 'type': '404'})
        )

    def test_signin_redirect(self):
        """
        Test redirect function which should redirect the user after a
        succesfull signin.

        """
        # Test with a requested redirect
        self.failUnlessEqual(signin_redirect(redirect='/accounts/'), '/accounts/')

        # Test with only the user specified
        user = get_user_model().objects.get(pk=1)
        self.failUnlessEqual(signin_redirect(user=user),
                             '/accounts/%s/' % user.username)

        # The ultimate fallback, probably never used
        self.failUnlessEqual(signin_redirect(), settings.LOGIN_REDIRECT_URL)

    def test_get_profile_model(self):
        """
        Test if the correct profile model is returned when
        ``get_profile_model()`` is called.

        """
        # A non existent model should also raise ``SiteProfileNotAvailable``
        # error.
        with self.settings(AUTH_PROFILE_MODULE='userena.FakeProfile'):
            self.assertRaises(SiteProfileNotAvailable, get_profile_model)

        # An error should be raised when there is no ``AUTH_PROFILE_MODULE``
        # supplied.
        with self.settings(AUTH_PROFILE_MODULE=None):
            self.assertRaises(SiteProfileNotAvailable, get_profile_model)

    def test_get_protocol(self):
        """ Test if the correct protocol is returned """
        self.failUnlessEqual(get_protocol(), 'http')

        with self.settings(USERENA_USE_HTTPS=True):
            self.failUnlessEqual(get_protocol(), 'https')



"""
Django accounts management made easy.

"""
default_app_config = 'userena.apps.UserenaConfig'

VERSION = (1, 5, 0)

__version__ = '.'.join((str(each) for each in VERSION[:4]))

def get_version():
    """
    Returns string with digit parts only as version.

    """
    return '.'.join((str(each) for each in VERSION[:3]))



# -*- coding: utf-8 -*-
from collections import defaultdict
import django


# this default dict will store all compat quirks for parameters of
# django.contrib.auth views
auth_views_compat_quirks = defaultdict(lambda: dict())

# below are quirks we must use because we can't change some userena API's
# like (url names)
if django.VERSION >= (1, 6, 0):  # pragma: no cover
    # in django >= 1.6.0 django.contrib.auth.views.reset no longer looks
    # for django.contrib.auth.views.password_reset_done but for
    # password_reset_done named url. To avoid duplicating urls we
    # provide custom post_reset_redirect
    auth_views_compat_quirks['userena_password_reset'] = {
        'post_reset_redirect': 'userena_password_reset_done',
    }

    # same case as above
    auth_views_compat_quirks['userena_password_reset_confirm'] = {
        'post_reset_redirect': 'userena_password_reset_complete',
    }
if django.VERSION >= (1, 7, 0):
    # Django 1.7 added a new argument to django.contrib.auth.views.password_reset
    # called html_email_template_name which allows us to pass it the html version
    # of the email
    auth_views_compat_quirks['html_email_template_name'] = 'userena/emails/password_reset_message.html'


# below are backward compatibility fixes
password_reset_uid_kwarg = 'uidb64'
if django.VERSION < (1, 6, 0):  # pragma: no cover
    # Django<1.6.0 uses uidb36, we construct urlpattern depending on this
    password_reset_uid_kwarg = 'uidb36'

if django.VERSION < (1, 5, 0):
    from django.utils.encoding import smart_unicode as smart_text
else:
    from django.utils.encoding import smart_text


# SiteProfileNotAvailable compatibility
if django.VERSION < (1, 7, 0):  # pragma: no cover
    from django.contrib.auth.models import SiteProfileNotAvailable
else:  # pragma: no cover
    class SiteProfileNotAvailable(Exception):
        pass

# old fallback to django hashcompat
try:
    from hashlib import sha1 as sha_constructor, md5 as md5_constructor
except ImportError:  # pragma: no cover
    from django.utils.hashcompat import sha_constructor, md5_constructor

if django.VERSION < (1, 7, 0):
    from django.db.models import get_model
else:
    from django.apps import apps
    get_model = apps.get_model



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='UserenaSignup',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('last_active', models.DateTimeField(help_text='The last date that the user was active.', null=True, verbose_name='last active', blank=True)),
                ('activation_key', models.CharField(max_length=40, verbose_name='activation key', blank=True)),
                ('activation_notification_send', models.BooleanField(default=False, help_text='Designates whether this user has already got a notification about activating their account.', verbose_name='notification send')),
                ('email_unconfirmed', models.EmailField(help_text='Temporary email address when the user requests an email change.', max_length=75, verbose_name='unconfirmed email address', blank=True)),
                ('email_confirmation_key', models.CharField(max_length=40, verbose_name='unconfirmed email verification key', blank=True)),
                ('email_confirmation_key_created', models.DateTimeField(null=True, verbose_name='creation date of email confirmation key', blank=True)),
                ('user', models.OneToOneField(related_name=b'userena_signup', verbose_name='user', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name': 'userena registration',
                'verbose_name_plural': 'userena registrations',
            },
            bases=(models.Model,),
        ),
    ]



from django.test import TestCase
from django.core.urlresolvers import reverse

from userena.utils import get_gravatar
from userena import settings as userena_settings

import re

class DecoratorTests(TestCase):
    """ Test the decorators """

    def test_secure_required(self):
        """
        Test that the ``secure_required`` decorator does a permanent redirect
        to a secured page.

        """
        with self.settings(USERENA_USE_HTTPS=True):
            response = self.client.get(reverse('userena_signin'))

            # Test for the permanent redirect
            self.assertEqual(response.status_code, 301)

            # Test if the redirected url contains 'https'. Couldn't use
            # ``assertRedirects`` here because the redirected to page is
            # non-existant.
            self.assertTrue('https' in response.get('Location'))
